@@ -57,7 +57,7 @@ HandleButtonEvents::HandleButtonEvents(QObject *parent)
     KGlobalAccel *accel = KGlobalAccel::self();
 
     QAction *globalAction = actionCollection->addAction("Sleep");
-    globalAction->setText(i18nc("@action:inmenu Global shortcut", "Sleep"));
+    globalAction->setText(i18nc("@action:inmenu Global shortcut", "Suspend"));
     accel->setGlobalShortcut(globalAction, Qt::Key_Sleep);
     connect(globalAction, SIGNAL(triggered(bool)), SLOT(suspendToRam()));
 
@@ -150,6 +150,10 @@ void HandleButtonEvents::processAction(uint action)
             // Turn off screen
             triggerAction("DPMSControl", QStringLiteral("TurnOff"));
             break;
+        case SuspendSession::ToggleScreenOnOffMode:
+            // Toggle screen on/off
+            triggerAction("DPMSControl", QStringLiteral("ToggleOnOff"));
+            break;
         default:
             triggerAction("SuspendSession", action);
             break;
