@@ -28,12 +28,12 @@
 #include <kwinkscreenhelpereffect.h>
 
 #include <KConfigGroup>
+#include <KIdleTime>
 #include <KLocalizedString>
 #include <KJob>
 
 #include <kworkspace.h>
 
-#include "screensaver_interface.h"
 #include <PowerDevilSettings.h>
 
 namespace PowerDevil
@@ -51,9 +51,8 @@ SuspendSession::SuspendSession(QObject* parent)
     setRequiredPolicies(PowerDevil::PolicyAgent::InterruptSession);
 
     connect(backend(), &PowerDevil::BackendInterface::resumeFromSuspend, this, [this]() {
-        // Notify the screensaver
-        OrgFreedesktopScreenSaverInterface iface("org.freedesktop.ScreenSaver", "/ScreenSaver", QDBusConnection::sessionBus());
-        iface.SimulateUserActivity();
+        KIdleTime::instance()->simulateUserActivity();
+
         PowerDevil::PolicyAgent::instance()->setupSystemdInhibition();
 
         m_fadeEffect->stop();
@@ -82,14 +81,22 @@ void SuspendSession::onProfileUnload()
 
 void SuspendSession::onWakeupFromIdle()
 {
-    // Nothing to do
+    m_fadeEffect->stop();
 }
 
 void SuspendSession::onIdleTimeout(int msec)
 {
-    Q_UNUSED(msec);
-    QVariantMap args;
-    args.insert("Type", m_autoType);
+    QVariantMap args{
+        {QStringLiteral("Type"), m_autoType}
+    };
+
+    // we fade the screen to black 5 seconds prior to suspending to alert the user
+    if (msec == m_idleTime - 5000) {
+        args.insert(QStringLiteral("GraceFade"), true);
+    } else {
+        args.insert(QStringLiteral("SkipFade"), true);
+    }
+
     trigger(args);
 }
 
@@ -98,23 +105,28 @@ void SuspendSession::onProfileLoad()
     // Nothing to do
 }
 
-void SuspendSession::triggerImpl(const QVariantMap& args)
+void SuspendSession::triggerImpl(const QVariantMap &args)
 {
-    qCDebug(POWERDEVIL) << "Triggered with " << args["Type"].toString() << args["SkipFade"].toBool();
+    qCDebug(POWERDEVIL) << "Suspend session triggered with" << args;
 
-    // Switch for screen fading
-    switch ((Mode) (args["Type"].toUInt())) {
-        case ToRamMode:
-        case ToDiskMode:
-        case SuspendHybridMode:
-            if (!args["SkipFade"].toBool()) {
-                m_savedArgs = args;
-                m_fadeEffect->start();
-                return;
-            }
-            break;
-        default:
-            break;
+    const auto mode = static_cast<Mode>(args["Type"].toUInt());
+
+    if (mode == ToRamMode || mode == ToDiskMode || mode == SuspendHybridMode) {
+        // don't suspend if shutting down
+        if (KWorkSpace::isShuttingDown()) {
+            qCDebug(POWERDEVIL) << "Not suspending because a shutdown is in progress";
+            return;
+        }
+
+        if (!args["SkipFade"].toBool()) {
+            m_savedArgs = args;
+            m_fadeEffect->start();
+            return;
+        }
+    }
+
+    if (args["GraceFade"].toBool()) {
+        return;
     }
 
     // Switch for real action
@@ -140,8 +152,10 @@ void SuspendSession::triggerImpl(const QVariantMap& args)
             break;
         case LockScreenMode: {
             // TODO should probably go through the backend (logind perhaps) eventually
-            OrgFreedesktopScreenSaverInterface iface("org.freedesktop.ScreenSaver", "/ScreenSaver", QDBusConnection::sessionBus());
-            iface.Lock();
+            QDBusConnection::sessionBus().asyncCall(QDBusMessage::createMethodCall("org.freedesktop.ScreenSaver",
+                                                                                   "/ScreenSaver",
+                                                                                   "org.freedesktop.ScreenSaver",
+                                                                                   "Lock"));
             break;
         }
         default:
@@ -158,7 +172,11 @@ bool SuspendSession::loadAction(const KConfigGroup& config)
 {
     if (config.isValid() && config.hasKey("idleTime") && config.hasKey("suspendType")) {
         // Add the idle timeout
-        registerIdleTimeout(config.readEntry<int>("idleTime", 0));
+        m_idleTime = config.readEntry<int>("idleTime", 0);
+        if (m_idleTime) {
+            registerIdleTimeout(m_idleTime - 5000);
+            registerIdleTimeout(m_idleTime);
+        }
         m_autoType = config.readEntry<uint>("suspendType", 0);
     }
 
@@ -182,13 +200,11 @@ void SuspendSession::suspendToRam()
 
 void SuspendSession::triggerSuspendSession(uint action)
 {
-    QVariantMap args;
-    args["Type"] = action;
-    args["Explicit"] = true;
-    trigger(args);
+    trigger({
+        {QStringLiteral("Type"), action},
+        {QStringLiteral("Explicit"), true}
+    });
 }
 
 }
 }
-
-#include "suspendsession.moc"
